@@ -2,19 +2,16 @@
 # -*- coding: utf-8 -*-
 
 import copy
+import warnings
 import numpy as np
 import pandas as pd
-from covsirphy.util.error import deprecate
+from covsirphy.util.error import deprecate, ScenarioNotFoundError, UnExecutedError
 from covsirphy.util.plotting import line_plot, box_plot
-from covsirphy.cleaning.term import Term
-from covsirphy.cleaning.jhu_data import JHUData
-from covsirphy.cleaning.population import PopulationData
-from covsirphy.phase.phase_unit import PhaseUnit
-from covsirphy.phase.phase_series import PhaseSeries
 from covsirphy.analysis.param_tracker import ParamTracker
+from covsirphy.analysis.data_handler import DataHandler
 
 
-class Scenario(Term):
+class Scenario(DataHandler):
     """
     Scenario analysis.
 
@@ -24,108 +21,42 @@ class Scenario(Term):
         country (str): country name
         province (str or None): province name
         tau (int or None): tau value
-        auto_complement (bool): if True, the number of cases will be complemented.
+        auto_complement (bool): if True and necessary, the number of cases will be complemented
     """
 
     def __init__(self, jhu_data, population_data, country, province=None, tau=None, auto_complement=True):
-        # Population
-        population_data = self.ensure_instance(
-            population_data, PopulationData, name="population_data")
-        self.population = population_data.value(country, province=province)
-        # Records
-        self.jhu_data = self.ensure_instance(
-            jhu_data, JHUData, name="jhu_data")
-        # Area name
-        self.country = country
-        self.province = province or self.UNKNOWN
-        self.area = JHUData.area_name(country, province)
-        # tau value must be shared
+        super().__init__(
+            jhu_data, population_data, country, province=province, auto_complement=auto_complement)
         self.tau = self.ensure_tau(tau)
-        # Whether complement the number of cases or not
-        self._auto_complement = bool(auto_complement)
-        self._complemented = False
-        # Create {scenario_name: PhaseSeries} and set records
-        self.record_df = pd.DataFrame()
-        self._first_date = None
-        self._last_date = None
-        self._init_phase_series()
+        self._update_range(first_date=None, last_date=None)
 
     def __getitem__(self, key):
-        if key in self._series_dict:
-            return self._series_dict[key]
-        raise KeyError(f"{key} scenario is not registered.")
-
-    def __setitem__(self, key, value):
-        self.ensure_instance(value, PhaseSeries, name="value")
-        self._series_dict[key] = value
-
-    def _init_phase_series(self):
         """
-        Initialize dictionary of phase series and set records.
-        Only when auto-complement mode, complement records if necessary.
-        """
-        # Set records (complement records, if necessary)
-        if self._auto_complement:
-            self.complement()
-            if self.record_df.empty:
-                self.complement_reverse()
-        else:
-            self.complement_reverse()
-        if self.record_df.empty:
-            area = self.jhu_data.area_name(
-                self.country, province=self.province)
-            raise ValueError(
-                f"Records with 'Recovered > 0' in {area} are un-registered.")
-        # First/last date of the records
-        if self._first_date is None:
-            series = self.record_df.loc[:, self.DATE]
-            self._first_date = series.min().strftime(self.DATE_FORMAT)
-            self._last_date = series.max().strftime(self.DATE_FORMAT)
-        # Set main scenario
-        self._series_dict = {
-            self.MAIN: PhaseSeries(
-                self._first_date, self._last_date, self.population
-            )
-        }
-
-    def complement(self, interval=2, max_ignored=100):
-        """
-        Complement the number of recovered cases when not updated/reported.
+        Return the phase series object for the scenario.
 
         Args:
-            interval (int): expected update interval of the number of recovered cases [days]
-            max_ignored (int): Max number of recovered cases to be ignored [cases]
+            key (str): scenario name
+
+        Raises:
+            ScenarioNotFoundError: the scenario is not registered
 
         Returns:
-            covsirphy.Scenario: self
-
-        Notes:
-            If the number of recovered cases did not change
-            for more than @interval days after reached @max_ignored cases,
-            complement will be applied to the number of recovered cases.
+            covsirphy.PhaseSeries
         """
-        self.record_df, self._complemented = self.jhu_data.subset_complement(
-            country=self.country, province=self.province,
-            start_date=self._first_date, end_date=self._last_date, population=self.population,
-            interval=interval, max_ignored=max_ignored)
-        return self
+        if key in self._tracker_dict:
+            return self._tracker_dict[key].series
+        raise ScenarioNotFoundError(key)
 
-    def complement_reverse(self):
+    def __setitem__(self, key, value):
         """
-        Restore the raw records. Reverse method of covsirphy.Scenario.complement().
+        Register a phase series.
 
-        Returns:
-            covsirphy.Scenario: self
+        Args:
+            key (str): scenario name
+            value (covsirphy.PhaseSeries): phase series object
         """
-        self.record_df = self.jhu_data.subset(
-            country=self.country,
-            province=self.province,
-            start_date=self._first_date,
-            end_date=self._last_date,
-            population=self.population
-        )
-        self._complemented = False
-        return self
+        self._tracker_dict[key] = ParamTracker(
+            self.record_df, value, area=self.area, tau=self.tau)
 
     @property
     def first_date(self):
@@ -136,10 +67,7 @@ class Scenario(Term):
 
     @first_date.setter
     def first_date(self, date):
-        self.ensure_date_order(self._first_date, date, name="date")
-        self.ensure_date_order(date, self._last_date, name="date")
-        self._first_date = date
-        self._init_phase_series()
+        self._update_range(first_date=date)
 
     @property
     def last_date(self):
@@ -150,60 +78,66 @@ class Scenario(Term):
 
     @last_date.setter
     def last_date(self, date):
+        self._update_range(last_date=date)
+
+    def _update_range(self, first_date=None, last_date=None):
+        """
+        Set first/last dates of the records.
+        Records to analyse will be updated and all scenarios will be cleared.
+
+        Args:
+            first_date (str or None): the first date of the records
+            last_date (str or None): the last date of the records
+        """
+        if first_date is not None:
+            self._first_date = self._ensure_date_in_range(first_date)
+        if last_date is not None:
+            self._last_date = self._ensure_date_in_range(last_date)
+        super().init_records()
+        self._init_trackers()
+
+    def _ensure_date_in_range(self, date):
+        """
+        Ensure that the date is in the range of (the first date, the last date).
+
+        Args:
+            date (str): date, like 01Jan2020
+        """
         self.ensure_date_order(self._first_date, date, name="date")
         self.ensure_date_order(date, self._last_date, name="date")
-        self._last_date = date
-        self._init_phase_series()
+        return date
 
-    def records(self, show_figure=True, filename=None):
+    def _init_trackers(self):
         """
-        Return the records as a dataframe.
+        Initialize dictionary of trackers.
+        """
+        series = ParamTracker.create_series(
+            first_date=self._first_date, last_date=self._last_date, population=self.population)
+        tracker = ParamTracker(
+            record_df=self.record_df, phase_series=series, area=self.area, tau=self.tau)
+        self._tracker_dict = {self.MAIN: tracker}
+
+    def _tracker(self, name, template="Main"):
+        """
+        Ensure that the phases series is registered.
+        If not registered, copy the template phase series.
 
         Args:
-            show_figure (bool): if True, show the records as a line-plot.
-            filename (str): filename of the figure, or None (show figure)
-
-        Returns:
-            pandas.DataFrame
-                Index:
-                    reset index
-                Columns:
-                    - Date (pd.TimeStamp): Observation date
-                    - Confirmed (int): the number of confirmed cases
-                    - Infected (int): the number of currently infected cases
-                    - Fatal (int): the number of fatal cases
-                    - Recovered (int): the number of recovered cases (> 0)
-
-        Notes:
-            Records with Recovered > 0 will be selected.
-            If complement was performed by Scenario.complement() or Scenario(auto_complement=True),
-            "(complemented)" will be added to the title of the figure.
-        """
-        df = self.record_df.drop(self.S, axis=1)
-        if not show_figure:
-            return df
-        title = f"{self.area}: Cases over time{' (complemented)' if self._complemented else ''}"
-        line_plot(
-            df.set_index(self.DATE).drop(self.C, axis=1),
-            title,
-            y_integer=True,
-            filename=filename
-        )
-        return df
-
-    def _create_tracker(self, name):
-        """
-        Create a instance of covsirphy.ParamTracker.
-
-        Args:
-            name (str): scenario name
+            name (str): phase series name
+            template (str): name of template phase series
 
         Returns:
             covsirphy.ParamTracker
         """
-        return ParamTracker(
-            record_df=self.record_df, phase_series=self._ensure_name(name),
-            area=self.area, tau=self.tau)
+        # Registered
+        if name in self._tracker_dict:
+            return self._tracker_dict[name]
+        # Un-registered and create it
+        if template not in self._tracker_dict:
+            raise ScenarioNotFoundError(template)
+        tracker = copy.deepcopy(self._tracker_dict[template])
+        self._tracker_dict[name] = tracker
+        return tracker
 
     @deprecate(old="Scenario.add_phase()", new="Scenario.add()")
     def add_phase(self, **kwargs):
@@ -236,60 +170,46 @@ class Scenario(Term):
             - Tau will be fixed as the last phase's value.
             - kwargs: Default values are the parameter values of the last phase.
         """
-        series = self._ensure_name(name)
+        if end_date is not None:
+            self.ensure_date(end_date, name="end_date")
+        tracker = self._tracker(name)
         try:
-            self._series_dict[name].add(
+            tracker.add(
                 end_date=end_date, days=days, population=population,
-                model=model, tau=self.tau, **kwargs
-            )
+                model=model, **kwargs)
         except ValueError:
-            last_date = series.unit("last").end_date
-            s1 = f'For "{name}" scenario, @end_date needs to match "DDMmmYYYY" format'
+            last_date = tracker.series.unit("last").end_date
             raise ValueError(
-                f'{s1} and be over {last_date}. However, {end_date} was applied.') from None
+                f'@end_date must be over {last_date}. However, {end_date} was applied.') from None
+        self._tracker_dict[name] = tracker
         return self
 
-    def _ensure_name(self, name, template="Main"):
-        """
-        Ensure that the phases series is registered.
-        If not registered, copy the template phase series.
-
-        Args:
-            name (str): phase series name
-            template (str): name of template phase series
-        """
-        if name in self._series_dict.keys():
-            return self._series_dict[name]
-        # Phase series
-        if template not in self._series_dict:
-            raise KeyError(f"Template scenario {template} does not exist.")
-        series = copy.deepcopy(self._series_dict[template])
-        self._series_dict[name] = series
-        return series
-
     def clear(self, name="Main", include_past=False, template="Main"):
         """
         Clear phase information.
 
         Args:
-            name (str): phase series name
-                - if 'Main', main phase series will be used
-                - if not registered, new phaseseries will be created
-            include_past (bool):
-                - if True, include past phases.
-                - future phase are always included
-            template (str): name of template phase series
+            name (str): scenario name
+            include_past (bool): if True, past phases will be removed as well as future phases
+            template (str): name of template scenario
 
         Returns:
             covsirphy.Scenario: self
+
+        Notes:
+            If un-registered scenario name was specified, new scenario will be created.
+            Future phases will be always deleted.
         """
-        self._ensure_name(name, template=template)
-        self._series_dict[name].clear(include_past=include_past)
+        tracker = self._tracker(name, template=template)
+        if include_past:
+            self[name] = tracker.delete_all()
+        else:
+            self[name] = tracker.delete(phases=tracker.future_phases()[0])
         return self
 
     def _delete_series(self, name):
         """
-        Delete a phase series.
+        Delete a scenario or initialise main scenario.
 
         Args:
             name (str): name of phase series
@@ -298,9 +218,9 @@ class Scenario(Term):
             covsirphy.Scenario: self
         """
         if name == self.MAIN:
-            self.clear(name=name, include_past=True)
-            return self
-        self._series_dict.pop(name)
+            self[self.MAIN] = self._tracker(self.MAIN).delete_all()
+        else:
+            self._tracker_dict.pop(name)
         return self
 
     def delete(self, phases=None, name="Main"):
@@ -320,20 +240,12 @@ class Scenario(Term):
             If the last phase is included in @phases, the dates will be released from phases.
             If the last phase is not included, the dates will be assigned to the previous phase.
         """
-        self._ensure_name(name)
         # Clear main series or delete sub phase series
         if phases is None:
             return self._delete_series(name)
         # Delete phases
-        if not isinstance(phases, list):
-            raise TypeError("@phases mut be a list of phase names.")
-        phases = list(set(phases))
-        if "last" in phases:
-            self._series_dict[name].delete("last")
-            phases.remove("last")
-        phases = sorted(phases, key=self.str2num, reverse=True)
-        for phase in phases:
-            self._series_dict[name].delete(phase)
+        tracker = self._tracker(name)
+        self[name] = tracker.delete(phases=phases)
         return self
 
     def disable(self, phases, name="Main"):
@@ -347,8 +259,7 @@ class Scenario(Term):
         Returns:
             covsirphy.Scenario: self
         """
-        tracker = self._create_tracker(name)
-        self._series_dict[name] = tracker.disable(phases)
+        self[name] = self._tracker(name).disable(phases)
         return self
 
     def enable(self, phases, name="Main"):
@@ -362,8 +273,7 @@ class Scenario(Term):
         Returns:
             covsirphy.Scenario: self
         """
-        tracker = self._create_tracker(name)
-        self._series_dict[name] = tracker.enable(phases)
+        self[name] = self._tracker(name).enable(phases)
         return self
 
     def combine(self, phases, name="Main", population=None, **kwargs):
@@ -383,30 +293,8 @@ class Scenario(Term):
         Returns:
             covsirphy.Scenario: self
         """
-        series = self._ensure_name(name)
-        # Sort and check @phase is a list
-        if not isinstance(phases, list):
-            raise TypeError("@phases must be a list of phase names.")
-        phases = list(set(phases))
-        if "last" in phases:
-            last_phase = "last"
-            phases.remove("last")
-            phases = sorted(phases, key=self.str2num, reverse=False)
-        else:
-            phases = sorted(phases, key=self.str2num, reverse=False)
-            last_phase = phases[-1]
-        # Setting of the new phase
-        start_date = series.unit(phases[0]).start_date
-        end_date = series.unit(last_phase).end_date
-        population = population or series.unit(last_phase).population
-        new_unit = PhaseUnit(start_date, end_date, population)
-        new_unit.set_ode(**kwargs)
-        # Phases to keep
-        kept_units = [
-            unit for unit in series if unit < start_date or unit > end_date]
-        # Replace units
-        self._series_dict[name].replaces(
-            phase=None, new_list=kept_units + [new_unit], keep_old=False)
+        self[name] = self._tracker(name).combine(
+            phases=phases, population=population, **kwargs)
         return self
 
     def separate(self, date, name="Main", population=None, **kwargs):
@@ -423,9 +311,8 @@ class Scenario(Term):
         Returns:
             covsirphy.Scenario: self
         """
-        tracker = self._create_tracker(name)
-        self._series_dict[name] = tracker.separate(
-            date, population=population, **kwargs)
+        self[name] = self._tracker(name).separate(
+            date=date, population=population, **kwargs)
         return self
 
     def _summary(self, name=None):
@@ -446,18 +333,17 @@ class Scenario(Term):
             If 'Main' was used as @name, main PhaseSeries will be used.
         """
         if name is None:
-            if len(self._series_dict.keys()) > 1:
+            if len(self._tracker_dict.keys()) > 1:
                 dataframes = []
-                for (_name, series) in self._series_dict.items():
-                    summary_df = series.summary()
+                for (_name, tracker) in self._tracker_dict.items():
+                    summary_df = tracker.series.summary()
                     summary_df = summary_df.rename_axis(self.PHASE)
                     summary_df[self.SERIES] = _name
                     dataframes.append(summary_df.reset_index())
                 df = pd.concat(dataframes, ignore_index=True, sort=False)
                 return df.set_index([self.SERIES, self.PHASE])
             name = self.MAIN
-        series = self._ensure_name(name)
-        return series.summary()
+        return self._tracker(name).series.summary()
 
     def summary(self, columns=None, name=None):
         """
@@ -484,12 +370,7 @@ class Scenario(Term):
             all_cols = [col for col in all_cols if col not in self.EST_COLS]
             all_cols += self.EST_COLS
         columns = columns or all_cols
-        if not isinstance(columns, list):
-            raise TypeError("@columns must be None or a list of strings.")
-        if not set(columns).issubset(df.columns):
-            raise KeyError(
-                f"Un-registered columns were selected as @columns. Please use {', '.join(df.columns)}."
-            )
+        self.ensure_list(columns, candidates=all_cols, name="columns")
         df = df.loc[:, columns]
         return df.dropna(how="all", axis=1).fillna(self.UNKNOWN)
 
@@ -515,6 +396,9 @@ class Scenario(Term):
             )
         try:
             include_init_phase = kwargs.pop("include_init_phase")
+            warnings.warn(
+                "@include_init_phase was deprecated. Please use Scenario.disable('0th').",
+                DeprecationWarning, stacklevel=2)
         except KeyError:
             include_init_phase = True
         try:
@@ -522,12 +406,12 @@ class Scenario(Term):
         except KeyError:
             pass
         # S-R trend analysis
-        tracker = self._create_tracker(name)
-        self._series_dict[name] = tracker.trend(
+        tracker = self._tracker(name)
+        self[name] = tracker.trend(
             force=force, show_figure=show_figure, filename=filename, **kwargs)
         # Disable 0th phase, if necessary
         if not include_init_phase:
-            self._series_dict[name].disable("0th")
+            self[name] = tracker.disable(phases=["0th"])
         return self
 
     def estimate(self, model, phases=None, name="Main", n_jobs=-1, **kwargs):
@@ -551,8 +435,7 @@ class Scenario(Term):
         if self.TAU in kwargs:
             raise ValueError(
                 "@tau must be specified when scenario = Scenario(), and cannot be specified here.")
-        tracker = self._create_tracker(name)
-        self.tau, self._series_dict[name] = tracker.estimate(
+        self.tau, self[name] = self._tracker(name).estimate(
             model=model, phases=phases, n_jobs=n_jobs, **kwargs)
 
     def phase_estimator(self, phase, name="Main"):
@@ -566,11 +449,10 @@ class Scenario(Term):
         Return:
             covsirphy.Estimator: estimator of the phase
         """
-        estimator = self._series_dict[name].unit(phase).estimator
+        estimator = self._tracker_dict[name].series.unit(phase).estimator
         if estimator is None:
-            raise AttributeError(
-                f'Scenario.estimate(model, phases=["{phase}"], name={name}) must be done in advance.'
-            )
+            raise UnExecutedError(
+                f'Scenario.estimate(model, phases=["{phase}"], name={name})')
         return estimator
 
     def estimate_history(self, phase, name="Main", **kwargs):
@@ -623,15 +505,12 @@ class Scenario(Term):
                     - Province (str): province/prefecture/state name
                     - Variables of the model and dataset (int): Confirmed etc.
         """
-        tracker = self._create_tracker(name)
+        tracker = self._tracker(name)
         try:
             sim_df = tracker.simulate(y0_dict=y0_dict)
-        except ValueError:
-            raise ValueError(
-                "Phases should be registered with Scenario.trend() or Scenario.add() in advance.") from None
-        except NameError:
-            raise NameError(
-                "Parameter estimation should be done with Scenario.estimate() in advance.") from None
+        except UnExecutedError:
+            raise UnExecutedError(
+                "Scenario.trend() or Scenario.add(), and Scenario.estimate(model)") from None
         if not show_figure:
             return sim_df
         # Show figure
@@ -663,7 +542,7 @@ class Scenario(Term):
         Notes:
             If 'Main' was used as @name, main PhaseSeries will be used.
         """
-        df = self._ensure_name(name).summary()
+        df = self.summary(name=name)
         if param not in df.columns:
             raise KeyError(f"@param must be in {', '.join(df.columns)}.")
         if phase == "last":
@@ -684,7 +563,7 @@ class Scenario(Term):
         Raises:
             KeyError: targets are not in the columns of summary dataframe
         """
-        series = self._series_dict[name]
+        series = self._tracker_dict[name].series
         model_set = {unit.model for unit in series}
         model_set = model_set - set([None])
         parameters = self.flatten([m.PARAMETERS for m in model_set])
@@ -693,8 +572,8 @@ class Scenario(Term):
         selectable_set = set(selectable_cols)
         df = series.summary().replace(self.UNKNOWN, None)
         if not selectable_set.issubset(df.columns):
-            raise ValueError(
-                f"Scenario.estimate(model, phases=None, name={name}) must be done in advance.")
+            raise UnExecutedError(
+                f'Scenario.estimate(model, phases=None, name="{name}")')
         targets = [targets] if isinstance(targets, str) else targets
         targets = targets or selectable_cols
         if not set(targets).issubset(selectable_set):
@@ -728,7 +607,7 @@ class Scenario(Term):
         Notes:
             If 'Main' was used as @name, main PhaseSeries will be used.
         """
-        self._ensure_name(name)
+        self._tracker(name)
         # Select target to show
         df = self._param_history(targets, name)
         # Divide by the first phase parameters
@@ -774,7 +653,7 @@ class Scenario(Term):
                     - Fatal({date}): Fatal on the next date of the last phase
         """
         _dict = {}
-        for (name, _) in self._series_dict.items():
+        for (name, _) in self._tracker_dict.items():
             # Predict the number of cases
             df = self.simulate(name=name, y0_dict=y0_dict, show_figure=False)
             df = df.set_index(self.DATE)
@@ -823,7 +702,7 @@ class Scenario(Term):
                     - nth_Rt etc.: Rt value if the values are not the same values
         """
         df = self._describe(y0_dict=y0_dict)
-        if not with_rt or len(self._series_dict) == 1:
+        if not with_rt or len(self._tracker_dict) == 1:
             return df
         # History of reproduction number
         rt_df = self.summary().reset_index()
@@ -851,9 +730,6 @@ class Scenario(Term):
                     - day parameter values (float)
         """
         df = self.summary(name=name).replace(self.UNKNOWN, None)
-        if self.ODE not in df.columns:
-            raise ValueError(
-                f"Scenario.estimate(model, name={name}) must be done in advance.")
         # Date range to dates
         df[self.START] = pd.to_datetime(df[self.START])
         df[self.END] = pd.to_datetime(df[self.END])
@@ -922,7 +798,7 @@ class Scenario(Term):
         """
         dataframes = []
         append = dataframes.append
-        for name in self._series_dict.keys():
+        for name in self._tracker_dict.keys():
             df = self._track(name=name, y0_dict=y0_dict)
             df.insert(0, self.SERIES, name)
             append(df)
@@ -985,7 +861,7 @@ class Scenario(Term):
         else:
             ylabel = target
         title = f"{self.area}: {ylabel} over time"
-        tracker = self._create_tracker(self.MAIN)
+        tracker = self._tracker(self.MAIN)
         line_plot(
             df, title, ylabel=ylabel,
             h=1.0 if target == self.RT else None,
@@ -1010,7 +886,7 @@ class Scenario(Term):
             pandas.DataFrame
         """
         df = self._track_param(name=name)
-        model = self._series_dict[name].unit("last").model
+        model = self._tracker(name).series.unit("last").model
         cols = list(set(df.columns) & set(model.PARAMETERS))
         if params is not None:
             if not isinstance(params, (list, set)):
@@ -1023,7 +899,7 @@ class Scenario(Term):
             title = f"{self.area}: {model.NAME} parameter change rates over time (1.0 on {f_date})"
             ylabel = f"Value per that on {f_date}"
             title = f"{self.area}: {ylabel} over time"
-            tracker = self._create_tracker(self.MAIN)
+            tracker = self._tracker(self.MAIN)
             line_plot(
                 df, title, ylabel=ylabel, v=tracker.change_dates(),
                 math_scale=False, filename=filename)
@@ -1052,19 +928,18 @@ class Scenario(Term):
         # Control
         self.clear(name=control, include_past=True)
         self.trend(name=control, show_figure=False)
-        tracker = self._create_tracker(control)
         try:
-            tracker.separate(beginning_date)
+            self.separate(date=beginning_date, name=control)
         except ValueError:
             pass
         self.estimate(model, name=control, **est_kwargs)
         # Target
         self.clear(name=target, include_past=False, template=control)
         phases_changed = [
-            self.num2str(i) for (i, ph) in enumerate(self._series_dict[target])
+            self.num2str(i) for (i, ph) in enumerate(self._tracker(target).series)
             if ph >= beginning_date
         ]
-        self.delete(phases_changed, name=target)
+        self.delete(phases=phases_changed, name=target)
         self.add(name=target, **param_dict)
         self.estimate(model, name=target, **est_kwargs)
 
@@ -1089,9 +964,7 @@ class Scenario(Term):
             If @phases is None, all phases will be used.
             @phases and @past_days can not be specified at the same time.
         """
-        name_temp = f"{name}_temporally_for_scoring"
-        self.clear(name=name_temp, include_past=False, template=name)
-        tracker = self._create_tracker(name_temp)
+        tracker = self._tracker(name)
         if past_days is not None:
             if phases is not None:
                 raise ValueError(
@@ -1099,7 +972,7 @@ class Scenario(Term):
             past_days = self.ensure_natural_int(past_days, name="past_days")
             # Separate a phase, if possible
             beginning_date = self.date_change(
-                self.last_date, days=0 - past_days)
+                self._last_date, days=0 - past_days)
             try:
                 tracker.separate(date=beginning_date)
             except ValueError:
@@ -1107,11 +980,8 @@ class Scenario(Term):
             # Ge the list of target phases
             phases = [
                 self.num2str(num) for (num, unit)
-                in enumerate(self._series_dict[name_temp])
+                in enumerate(tracker.series)
                 if unit >= beginning_date
             ]
-        score = tracker.score(
+        return tracker.score(
             metrics=metrics, variables=variables, phases=phases, y0_dict=y0_dict)
-        if name_temp != name:
-            self.delete(name=name_temp)
-        return score
