@@ -3,8 +3,10 @@
 
 import numpy as np
 import pandas as pd
+from covsirphy.util.error import SubsetNotFoundError
 from covsirphy.cleaning.cbase import CleaningBase
 from covsirphy.cleaning.country_data import CountryData
+from covsirphy.cleaning.jhu_complement import JHUDataComplementHandler
 
 
 class JHUData(CleaningBase):
@@ -18,15 +20,23 @@ class JHUData(CleaningBase):
 
     def __init__(self, filename, citation=None):
         super().__init__(filename, citation)
-        self._closing_period = None
+        self._recovery_period = None
+
+    @property
+    def recovery_period(self):
+        """
+        int: expected value of recovery period [days]
+        """
+        return self._recovery_period
+
+    @recovery_period.setter
+    def recovery_period(self, value):
+        self._recovery_period = self.ensure_natural_int(value)
 
     def cleaned(self, **kwargs):
         """
         Return the cleaned dataset.
 
-        Note:
-            Cleaning method is defined by self._cleaning() method.
-
         Args:
             kwargs: keword arguments will be ignored.
 
@@ -42,13 +52,14 @@ class JHUData(CleaningBase):
                     - Infected (int): the number of currently infected cases
                     - Fatal (int): the number of fatal cases
                     - Recovered (int): the number of recovered cases
+
+        Notes:
+            Cleaning method is defined by self._cleaning() method.
         """
         if "population" in kwargs.keys():
             raise ValueError(
                 "@population was removed in JHUData.cleaned(). Please use JHUData.subset()")
-        df = self._cleaned_df.copy()
-        df = df.loc[:, self.COLUMNS]
-        return df
+        return self._cleaned_df.loc[:, self.COLUMNS]
 
     def _cleaning(self):
         """
@@ -74,12 +85,9 @@ class JHUData(CleaningBase):
         df = df.rename(
             {
                 "ObservationDate": self.DATE,
-                "ISO3": self.ISO3,
                 "Country/Region": self.COUNTRY,
                 "Province/State": self.PROVINCE,
-                "Confirmed": self.C,
                 "Deaths": self.F,
-                "Recovered": self.R
             },
             axis=1
         )
@@ -118,32 +126,77 @@ class JHUData(CleaningBase):
 
         Args:
             country_data (covsirphy.CountryData): dataset object of the country
+                Index: reset index
+                Columns:
+                    - Date (pd.TimeStamp): Observation date
+                    - Province (str): province name
+                    - Confirmed (int): the number of confirmed cases
+                    - Infected (int): the number of currently infected cases
+                    - Fatal (int): the number of fatal cases
+                    - Recovered (int): the number of recovered cases
+                    - The other columns will be ignored
 
         Returns:
             covsirphy.JHUData: self
+
+        Notes:
+            Citation of the country data will be added to 'JHUData.citation' description.
         """
         self.ensure_instance(country_data, CountryData, name="country_data")
         # Read new dataset
         country = country_data.country
-        new = country_data.cleaned()
+        new = country_data.cleaned().loc[:, self.COLUMNS]
         new[self.ISO3] = self.country_to_iso3(country)
         # Remove the data in the country from JHU dataset
         df = self._cleaned_df.copy()
-        df = df.loc[df[self.COUNTRY] != country, :]
+        df = df.loc[df[self.COUNTRY] != country]
         # Combine JHU data and the new data
         df = pd.concat([df, new], axis=0, sort=False)
         self._cleaned_df = df.copy()
+        # Citation
+        self._citation += f"\n{country_data.citation}"
         return self
 
-    def _subset(self, country, province, start_date, end_date, population):
+    def _subset(self, country, province, start_date, end_date):
         """
-        Return the subset of dataset.
+        Return the subset of dataset or empty dataframe (when no records were found).
 
         Args:
             country (str): country name or ISO3 code
             province (str or None): province name
             start_date (str or None): start date, like 22Jan2020
             end_date (str or None): end date, like 01Feb2020
+
+        Returns:
+            pandas.DataFrame
+                Index:
+                    reset index
+                Columns:
+                    - Date (pd.TimeStamp): Observation date
+                    - Confirmed (int): the number of confirmed cases
+                    - Infected (int): the number of currently infected cases
+                    - Fatal (int): the number of fatal cases
+                    - Recovered (int): the number of recovered cases
+        """
+        try:
+            return super().subset(
+                country=country, province=province, start_date=start_date, end_date=end_date)
+        except SubsetNotFoundError:
+            return pd.DataFrame(columns=self.NLOC_COLUMNS)
+
+    def _calculate_susceptible(self, subset_df, population):
+        """
+        Return the subset of dataset.
+
+        Args:
+            subset_df (pandas.DataFrame)
+                Index: reset index
+                Columns:
+                    - Date (pd.TimeStamp): Observation date
+                    - Confirmed (int): the number of confirmed cases
+                    - Infected (int): the number of currently infected cases
+                    - Fatal (int): the number of fatal cases
+                    - Recovered (int): the number of recovered cases
             population (int or None): population value
 
         Returns:
@@ -155,24 +208,16 @@ class JHUData(CleaningBase):
                     - Confirmed (int): the number of confirmed cases
                     - Infected (int): the number of currently infected cases
                     - Fatal (int): the number of fatal cases
-                    - Recovered (int): the number of recovered cases (> 0)
+                    - Recovered (int): the number of recovered cases
                     - Susceptible (int): the number of susceptible cases, if calculated
 
         Notes:
             If @population is not None, the number of susceptible cases will be calculated.
         """
-        # Subset with area and start/end date
-        try:
-            df = super().subset(
-                country=country, province=province, start_date=start_date, end_date=end_date)
-        except KeyError:
-            columns = self.NLOC_COLUMNS if population is None else self.SUB_COLUMNS
-            return pd.DataFrame(columns=columns)
-        # Calculate Susceptible if population value was applied
         if population is None:
-            return df
-        df.loc[:, self.S] = population - df.loc[:, self.C]
-        return df
+            return subset_df
+        subset_df.loc[:, self.S] = population - subset_df.loc[:, self.C]
+        return subset_df[self.SUB_COLUMNS]
 
     def subset(self, country, province=None, start_date=None, end_date=None, population=None):
         """
@@ -200,31 +245,28 @@ class JHUData(CleaningBase):
         Notes:
             If @population is not None, the number of susceptible cases will be calculated.
         """
-        area = self.area_name(country, province=province)
+        country_alias = self.ensure_country_name(country)
         # Subset with area, start/end date and calculate Susceptible
         subset_df = self._subset(
-            country=country, province=province,
-            start_date=start_date, end_date=end_date, population=population)
+            country=country, province=province, start_date=start_date, end_date=end_date)
         if subset_df.empty:
-            area = self.area_name(country, province=province)
-            s_fr = "" if start_date is None else f" from {start_date}"
-            s_to = "" if end_date is None else f" from {end_date}"
-            raise KeyError(f"Records in {area}{s_fr}{s_to} are un-registered.")
+            raise SubsetNotFoundError(
+                country=country, country_alias=country_alias, province=province,
+                start_date=start_date, end_date=end_date)
+        # Calculate Susceptible
+        df = self._calculate_susceptible(subset_df, population)
         # Select records where Recovered > 0
-        df = subset_df.loc[subset_df[self.R] > 0, :]
-        df = df.reset_index(drop=True)
+        df = df.loc[df[self.R] > 0, :].reset_index(drop=True)
         if df.empty:
-            series = subset_df[self.DATE]
-            start_date = start_date or series.min().strftime(self.DATE_FORMAT)
-            end_date = end_date or series.max().strftime(self.DATE_FORMAT)
-            area = self.area_name(country, province=province)
-            raise ValueError(
-                f"Records with 'Recovered > 0' in {area} from {start_date} to {end_date} are un-registered.")
+            raise SubsetNotFoundError(
+                country=country, country_alias=country_alias, province=province,
+                start_date=start_date, end_date=end_date, message="with 'Recovered > 0'") from None
         return df
 
     def to_sr(self, country, province=None,
               start_date=None, end_date=None, population=None):
         """
+        Create Susceptible/Recovered dataset without complement.
 
         Args:
             country (str): country name
@@ -248,8 +290,7 @@ class JHUData(CleaningBase):
         population = self.ensure_population(population)
         subset_df = self.subset(
             country=country, province=province,
-            start_date=start_date, end_date=end_date, population=population
-        )
+            start_date=start_date, end_date=end_date, population=population)
         return subset_df.set_index(self.DATE).loc[:, [self.R, self.S]]
 
     @classmethod
@@ -258,7 +299,7 @@ class JHUData(CleaningBase):
         Create JHUData instance using a pandas dataframe.
 
         Args:
-            dataframe (pd.DataFrame): Cleaned dataset
+            dataframe (pd.DataFrame): cleaned dataset
 
         Returns:
             covsirphy.JHUData: JHU-style dataset
@@ -328,12 +369,17 @@ class JHUData(CleaningBase):
 
         Returns:
             int: closing period [days]
+
+        Notes:
+            If no records we can use for calculation were registered, 12 [days] will be applied.
         """
         # Get cleaned dataset at country level
         df = self._cleaned_df.copy()
         df = df.loc[df[self.PROVINCE] == self.UNKNOWN]
         # Select records of countries where recovered values are reported
         df = df.groupby(self.COUNTRY).filter(lambda x: x[self.R].sum() != 0)
+        if df.empty:
+            return 12
         # Total number of confirmed/closed cases of selected records
         df = df.groupby(self.DATE).sum()
         df[self.FR] = df[[self.F, self.R]].sum(axis=1)
@@ -347,151 +393,7 @@ class JHUData(CleaningBase):
         df["Elapsed"] = df[self.FR] - df[self.C]
         df = df.loc[df["Elapsed"] > 0]
         # Calculate mode value of closing period
-        return df["Elapsed"].mode().astype(np.int64).values[0]
-
-    def _complement_non_monotonic(self, subset_df, monotonic_columns):
-        """
-        Make the number of cases show monotonic increasing, if necessary.
-
-        Args:
-            subset_df (pandas.DataFrame): subset records with country, province and start/end dates
-                Index:
-                    reset index
-                Columns:
-                    - Date (pd.TimeStamp): Observation date
-                    - Confirmed (int): the number of confirmed cases
-                    - Fatal (int): the number of fatal cases
-                    - the other columns will be ignored
-            monotonic_columns (list[str]): columns that show monotonic increasing
-
-        Returns:
-            pandas.DataFrame
-                Index:
-                    reset index
-                Columns:
-                    - Date (pd.TimeStamp): Observation date
-                    - Confirmed (int): the number of confirmed cases
-                    - Infected (int): the number of currently infected cases
-                    - Fatal (int): the number of fatal cases
-                    - Recovered (int): the number of recovered cases
-                    - Susceptible (int): the number of susceptible cases, if @subset_df has
-        """
-        # If all variables show monotic increasing, complement will not be done
-        columns = [
-            col for col in monotonic_columns if not subset_df[col].is_monotonic_increasing]
-        if not columns:
-            return subset_df
-        # Complement
-        df = subset_df.set_index(self.DATE)
-        for col in columns:
-            decreased_dates = df[df[col].diff() < 0].index.tolist()
-            for date in decreased_dates:
-                # Raw value on the decreased date
-                raw_last = df.loc[date, col]
-                # Extrapolated value on the date
-                series = df.loc[:date, col]
-                series.iloc[-1] = None
-                series.interpolate(method="spline", order=1, inplace=True)
-                series.fillna(method="ffill", inplace=True)
-                # Reduce values to the previous date
-                df.loc[:date, col] = series * raw_last / series.iloc[-1]
-                df[col] = df[col].astype(np.int64)
-        # Calculate Infected
-        df[self.CI] = df[self.C] - df[self.F] - df[self.R]
-        return df.reset_index()
-
-    def _complement_recovered_full(self, subset_df, max_ignored):
-        """
-        Estimate the number of recovered cases with closing period, if necessary.
-
-        Args:
-            subset_df (pandas.DataFrame): subset records with country, province and start/end dates
-                Index:
-                    reset index
-                Columns:
-                    - Date (pd.TimeStamp): Observation date
-                    - Confirmed (int): the number of confirmed cases
-                    - Fatal (int): the number of fatal cases
-                    - the other columns will be ignored
-            max_ignored (int): Max number of recovered cases to be ignored [cases]
-
-        Returns:
-            pandas.DataFrame
-                Index:
-                    reset index
-                Columns:
-                    - Date (pd.TimeStamp): Observation date
-                    - Confirmed (int): the number of confirmed cases
-                    - Infected (int): the number of currently infected cases
-                    - Fatal (int): the number of fatal cases
-                    - Recovered (int): the number of recovered cases
-                    - Susceptible (int): the number of susceptible cases, if @subset_df has
-        """
-        c, f, r = subset_df[[self.C, self.F, self.R]].max().tolist()
-        if r > max_ignored and r > (c - f) * 0.1:
-            return subset_df
-        df = subset_df.set_index(self.DATE)
-        # Closing period
-        self._closing_period = self._closing_period or self.calculate_closing_period()
-        # Estimate recovered records
-        shifted = df[self.C].shift(periods=self._closing_period, freq="D")
-        df[self.R] = shifted - df[self.F]
-        df.loc[df[self.R] < 0, self.R] = None
-        df[self.R].interpolate(method="spline", order=1, inplace=True)
-        df[self.R] = df[self.R].fillna(0).astype(np.int64)
-        df = self._complement_non_monotonic(
-            df.reset_index(), [self.R]).set_index(self.DATE)
-        # Re-calculate infected records
-        df[self.CI] = df[self.C] - df[self.F] - df[self.R]
-        return df.reset_index()
-
-    def _complement_recovered_partial(self, subset_df, interval, max_ignored):
-        """
-        Complement the number of recovered cases when not updated, if necessary.
-
-        Args:
-            subset_df (pandas.DataFrame): subset records with country, province and start/end dates
-                Index:
-                    reset index
-                Columns:
-                    - Date (pd.TimeStamp): Observation date
-                    - Confirmed (int): the number of confirmed cases
-                    - Fatal (int): the number of fatal cases
-                    - the other columns will be ignored
-            interval (int): expected update interval of the number of recovered cases [days]
-            max_ignored (int): Max number of recovered cases to be ignored [cases]
-
-        Returns:
-            pandas.DataFrame
-                Index:
-                    reset index
-                Columns:
-                    - Date (pd.TimeStamp): Observation date
-                    - Confirmed (int): the number of confirmed cases
-                    - Infected (int): the number of currently infected cases
-                    - Fatal (int): the number of fatal cases
-                    - Recovered (int): the number of recovered cases
-                    - Susceptible (int): the number of susceptible cases, if @subset_df has
-
-        Notes:
-            If the number of recovered cases did not change
-            for more than @interval days after reached @max_ignored cases,
-            complement will be applied to the number of recovered cases.
-        """
-        df = subset_df.copy()
-        # If updated, do not perform complement
-        series = df.loc[df[self.R] > max_ignored, self.R]
-        max_frequency = series.value_counts().max()
-        if max_frequency <= interval:
-            return subset_df
-        # Complement
-        df.loc[df.duplicated([self.R], keep="last"), self.R] = None
-        df[self.R].interpolate(
-            method="linear", inplace=True, limit_direction="both")
-        df[self.R] = df[self.R].fillna(method="bfill").round().astype(np.int64)
-        # Calculate Infected
-        df[self.CI] = df[self.C] - df[self.F] - df[self.R]
-        return df
+        return int(df["Elapsed"].mode().astype(np.int64).values[0])
 
     def subset_complement(self, country, province=None,
                           start_date=None, end_date=None, population=None,
@@ -510,7 +412,7 @@ class JHUData(CleaningBase):
             max_ignored (int): Max number of recovered cases to be ignored [cases]
 
         Returns:
-            tuple(pandas.DataFrame, bool):
+            tuple(pandas.DataFrame, str/bool):
                 pandas.DataFrame:
                     Index:
                         reset index
@@ -521,32 +423,82 @@ class JHUData(CleaningBase):
                         - Fatal(int): the number of fatal cases
                         - Recovered (int): the number of recovered cases ( > 0)
                         - Susceptible(int): the number of susceptible cases, if calculated
-                bool: whether recovered data complemented or not
+                str/bool: kind of complement or False
 
         Notes:
-            If @ population is not None, the number of susceptible cases will be calculated.
+            If @population is not None, the number of susceptible cases will be calculated.
         """
         # Arguments
         interval = self.ensure_natural_int(interval, name="interval")
         max_ignored = self.ensure_natural_int(max_ignored, name="max_ignored")
         # Subset with area, start/end date and calculate Susceptible
+        country_alias = self.ensure_country_name(country)
         subset_df = self._subset(
-            country=country, province=province,
-            start_date=start_date, end_date=end_date, population=population)
+            country=country, province=province, start_date=start_date, end_date=end_date)
         if subset_df.empty:
-            area = self.area_name(country, province=province)
-            s_fr = "" if start_date is None else f" from {start_date}"
-            s_to = "" if end_date is None else f" from {end_date}"
-            raise KeyError(f"Records in {area}{s_fr}{s_to} are un-registered.")
-        # Complement recovered value if necessary
-        df = self._complement_non_monotonic(
-            subset_df, monotonic_columns=self.MONO_COLUMNS)
-        df = self._complement_recovered_full(df, max_ignored=max_ignored)
-        df = self._complement_recovered_partial(
-            df, interval=interval, max_ignored=max_ignored)
-        # Whether complemented or not
-        is_complemented = not df.equals(subset_df)
+            raise SubsetNotFoundError(
+                country=country, country_alias=country_alias, province=province,
+                start_date=start_date, end_date=end_date) from None
+        # Complement, if necessary
+        handler = JHUDataComplementHandler(
+            recovery_period=self._recovery_period or self.calculate_closing_period(),
+            interval=interval,
+            max_ignored=max_ignored,
+        )
+        df, status = handler.run(subset_df)
+        # Calculate Susceptible
+        df = self._calculate_susceptible(df, population)
+        # Kind of complement or False
+        is_complemented = status or False
         # Select records where Recovered > 0
-        df = df.loc[df[self.R] > 0, :]
-        df = df.reset_index(drop=True)
+        df = df.loc[df[self.R] > 0, :].reset_index(drop=True)
         return (df, is_complemented)
+
+    def records(self, country, province=None, start_date=None, end_date=None, population=None,
+                auto_complement=True, **kwargs):
+        """
+        JHU-style dataset for the area from the start date to the end date.
+        Records with Recovered > 0 will be selected.
+
+        Args:
+            country(str): country name or ISO3 code
+            province(str or None): province name
+            start_date(str or None): start date, like 22Jan2020
+            end_date(str or None): end date, like 01Feb2020
+            population(int or None): population value
+            auto_complement (bool): if True and necessary, the number of cases will be complemented
+            kwargs: the other arguments of JHUData.subset_complement()
+
+        Returns:
+            tuple(pandas.DataFrame, bool):
+                pandas.DataFrame:
+                    Index: reset index
+                    Columns:
+                        - Date(pd.TimeStamp): Observation date
+                        - Confirmed(int): the number of confirmed cases
+                        - Infected(int): the number of currently infected cases
+                        - Fatal(int): the number of fatal cases
+                        - Recovered (int): the number of recovered cases ( > 0)
+                        - Susceptible(int): the number of susceptible cases, if calculated
+                str/bool: kind of complement or False
+
+        Notes:
+            If @ population is not None, the number of susceptible cases will be calculated.
+            If necessary and @auto_complement is True, complement recovered data.
+        """
+        country_alias = self.ensure_country_name(country)
+        subset_arg_dict = {
+            "country": country, "province": province,
+            "start_date": start_date, "end_date": end_date, "population": population,
+        }
+        if auto_complement:
+            df, is_complemented = self.subset_complement(
+                **subset_arg_dict, **kwargs)
+            if not df.empty:
+                return (df, is_complemented)
+        try:
+            return (self.subset(**subset_arg_dict), False)
+        except ValueError:
+            raise SubsetNotFoundError(
+                country=country, country_alias=country_alias, province=province,
+                start_date=start_date, end_date=end_date, message="with 'Recovered > 0'") from None
