@@ -5,7 +5,7 @@ from datetime import datetime
 from dask import dataframe as dd
 import numpy as np
 import pandas as pd
-from covsirphy.util.error import deprecate
+from covsirphy.util.error import deprecate, SubsetNotFoundError
 from covsirphy.cleaning.cbase import CleaningBase
 
 
@@ -60,7 +60,6 @@ class PopulationData(CleaningBase):
                 "Country/Region": self.COUNTRY,
                 "Province.State": self.PROVINCE,
                 "Province/State": self.PROVINCE,
-                "Population": self.N,
                 "ObservationDate": self.DATE
             },
             axis=1
@@ -91,8 +90,8 @@ class PopulationData(CleaningBase):
         df = df.dropna(subset=[self.N]).reset_index(drop=True)
         df[self.N] = df[self.N].astype(np.int64)
         # Columns to use
-        df = df.loc[:, [self.ISO3, self.COUNTRY,
-                        self.PROVINCE, self.DATE, self.N]]
+        df = df.loc[
+            :, [self.ISO3, self.COUNTRY, self.PROVINCE, self.DATE, self.N]]
         # Remove duplicates
         df = df.drop_duplicates().reset_index(drop=True)
         return df
@@ -154,14 +153,13 @@ class PopulationData(CleaningBase):
         Notes:
             If @date is None, the created date of the instancewill be used
         """
+        country_alias = self.ensure_country_name(country)
         try:
             df = self.subset(
                 country=country, province=province, start_date=date, end_date=date)
         except KeyError:
-            area = self.area_name(country, province=province)
-            date_str = "" if date is None else " on {date}"
-            raise KeyError(
-                f"Records in {area}{date_str} are un-registered.") from None
+            raise SubsetNotFoundError(
+                country=country, country_alias=country_alias, province=province, date=date)
         df = df.sort_values(self.DATE)
         return int(df.loc[df.index[-1], [self.N]].values[0])
 
