@@ -34,12 +34,12 @@ class SIRD(ModelBase):
     VARS_INCLEASE = [ModelBase.R, ModelBase.F]
     # Example set of parameters and initial values
     EXAMPLE = {
-        "step_n": 180,
-        "population": 1_000_000,
-        "param_dict": {
+        ModelBase.STEP_N: 180,
+        ModelBase.N.lower(): 1_000_000,
+        ModelBase.PARAM_DICT: {
             "kappa": 0.005, "rho": 0.2, "sigma": 0.075,
         },
-        "y0_dict": {
+        ModelBase.Y0_DICT: {
             ModelBase.S: 999_000, ModelBase.CI: 1000, ModelBase.R: 0, ModelBase.F: 0,
         },
     }
@@ -96,6 +96,7 @@ class SIRD(ModelBase):
         df = cls.ensure_dataframe(
             taufree_df, name="taufree_df", columns=[cls.TS, *cls.VARIABLES]
         )
+        df = df.loc[(df[cls.S] > 0) & (df[cls.CI] > 0)]
         n, t = population, df[cls.TS]
         s, i, r, d = df[cls.S], df[cls.CI], df[cls.R], df[cls.F]
         # kappa = (dD/dt) / I
@@ -106,7 +107,7 @@ class SIRD(ModelBase):
         sigma_series = r.diff() / t.diff() / i
         # Calculate quantile
         _dict = {
-            k: v.quantile(cls.QUANTILE_RANGE)
+            k: tuple(v.quantile(cls.QUANTILE_RANGE).clip(0, 1))
             for (k, v) in zip(
                 ["kappa", "rho", "sigma"],
                 [kappa_series, rho_series, sigma_series]
